package org.fluff.splodge.test;

import java.io.*;

import org.fluff.splodge.scim.*;

/**

23rd June 2004, 09:49 -> 10:05

@author Phil Norman
*/

public class Client {
	public static void main(String[] argv) {
		if (argv.length != 2) {
			System.err.println("Usage: Client <hostname> <port number>");
			System.exit(1);
		}
		String hostname = argv[0];
		int port = Integer.parseInt(argv[1]);
		try {
			TestServer server = (TestServer) SCIMClient.connect(hostname, port);
			DirLister lister = server.getDirLister();
			mainLoop(lister);
		} catch (IOException ex) {
			System.err.println("Failed to connect to remote server on " + hostname + ":" + port + ".");
			ex.printStackTrace();
		}
	}
	
	private static void mainLoop(DirLister lister) throws IOException {
		BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
		String line;
		while ((line = in.readLine()) != null) {
			line = line.trim();
			try {
				if (line.startsWith("help")) {
					printHelp();
				} else if (line.startsWith("cd")) {
					lister.changeDirectory(getEntry(line));
				} else if (line.equals("ls")) {
					listFiles(lister);
				} else if (line.startsWith("isdir")) {
					String entry = getEntry(line);
					System.out.println(entry + (lister.isDirectory(entry) ? " is" : " is not") + " a directory.");
				} else if (line.startsWith("size")) {
					String entry = getEntry(line);
					System.out.println(entry + " has size " + lister.getSize(entry));
				} else if (line.equals("up")) {
					lister.upDirectory();
				} else if (line.equals("pwd")) {
					System.out.println("Path: " + lister.getPath());
				}
			} catch (IllegalArgumentException ex) {
				System.out.println("Bad syntax in line \"" + line + "\": " + ex.getMessage());
			}
		}
		System.exit(0);
	}
	
	private static void printHelp() {
		System.out.println("Available commands:");
		System.out.println("    help: show this message.");
		System.out.println("    cd <dir>: change directory into the argument.");
		System.out.println("    ls: list the contents of the current directory.");
		System.out.println("    isdir <entry>: report whether the specified entry is a directory.");
		System.out.println("    size <entry>: report the size of the specified entry.");
		System.out.println("    up: change directory one level up.");
		System.out.println("    pwd: show the current directory path.");
	}
	
	private static String getEntry(String line) {
		int index = line.indexOf(' ');
		if (index == -1) {
			throw new IllegalArgumentException("No argument found.");
		} else {
			return line.substring(index).trim();
		}
	}
	
	private static void listFiles(DirLister lister) {
		String[] files = lister.getDirectoryListing();
		System.out.println("Contents of directory:");
		for (int i = 0; i < files.length; i++) {
			System.out.println("    " + files[i]);
		}
	}
}
