package org.fluff.splodge.util;

import java.lang.ref.*;
import java.util.*;

/**
The ReferenceNotifier is a singleton class.  Listeners may be registered with objects whose
garbage collection is interesting to know about, and they are called when the object gets
collected.

Started: 22nd June 2004, 11:46
Finished: 22nd June 2004, 11:52

@author Phil Norman
*/

public class ReferenceNotifier implements Runnable {
	private static final ReferenceNotifier INSTANCE = new ReferenceNotifier();
	
	private ReferenceQueue queue = new ReferenceQueue();
	private Map listeners = Collections.synchronizedMap(new HashMap());
	
	private ReferenceNotifier() {
		(new Thread(this, "Reference Notifier")).start();
	}
	
	public static ReferenceNotifier getInstance() {
		return INSTANCE;
	}
	
	public void addListener(Object instance, ReferenceListener listener) {
		WeakReference ref = new WeakReference(instance, queue);
		listeners.put(ref, listener);
	}
	
	public void run() {
		while (true) {
			try {
				Reference ref = queue.remove();
				ReferenceListener listener = (ReferenceListener) listeners.get(ref);
				if (listener != null) {
					listener.objectDestroyed();
				}
			} catch (Exception ex) {
				ex.printStackTrace();
			}
		}
	}
}
